package swagger;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.net.URL;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.nio.file.Paths;

import org.apache.commons.io.output.FileWriterWithEncoding;

import io.github.swagger2markup.GroupBy;
import io.github.swagger2markup.Language;
import io.github.swagger2markup.Swagger2MarkupConfig;
import io.github.swagger2markup.Swagger2MarkupConverter;
import io.github.swagger2markup.builder.Swagger2MarkupConfigBuilder;
import io.github.swagger2markup.markup.builder.MarkupLanguage;

public class DocExport {

	public static void genConfluenceDocs(String group, String groupName) throws Exception {
		// 输出Confluence使用的格式
		Swagger2MarkupConfig config = new Swagger2MarkupConfigBuilder()
				.withMarkupLanguage(MarkupLanguage.CONFLUENCE_MARKUP).withOutputLanguage(Language.ZH)
				.withPathsGroupedBy(GroupBy.TAGS).withGeneratedExamples().withoutInlineSchema().build();

		Swagger2MarkupConverter
				.from(new URL("http://localhost:8088/v2/api-docs?group=" + URLEncoder.encode(groupName, "utf8")))
				.withConfig(config).build().toFolder(Paths.get("./docs/confluence/generated/" + group));
	}

	public static void getListOfApiUrls(String group) throws IOException {
		String path = "./docs/confluence/generated/" + group;
		File pathFile = new File(path + "/paths.txt");
		new File("./docs/output").mkdir();
		File outFile = new File("./docs/output/api-" + group + ".csv");
		try (BufferedReader bufferedReader = new BufferedReader(new FileReader(pathFile));
				BufferedWriter bufferedWriter = new BufferedWriter(
						new FileWriterWithEncoding(outFile, StandardCharsets.UTF_8))) {
			writeLineToCSV(new String[] { "类描述", "类名", "接口描述", "接口类型", "接口地址" }, bufferedWriter);
			bufferedWriter.newLine();
			String line = null, className = "", classDesc = "", apiDesc = "", apiMethodType = "", apiUrl = "";
			boolean controllerName = false, outputApiLine = false;
			while ((line = bufferedReader.readLine()) != null) {
				if (controllerName) {
					className = line.replace(" ", "");
					controllerName = false;
					continue;
				} else if (line.startsWith("h3")) {
					classDesc = line.split(" ")[1];
					controllerName = true;
				} else if (line.startsWith("h4")) {
					apiDesc = line.split(" ")[1].replaceAll(",", "、");
				} else if (line.startsWith("GET") || line.startsWith("POST") || line.startsWith("PUT")
						|| line.startsWith("DELETE")) {
					String api[] = line.split(" ");
					apiMethodType = api[0];
					apiUrl = api[1];
					outputApiLine = true;
				}
				if (outputApiLine) {
					writeLineToCSV(new String[] { classDesc, className, apiDesc, apiMethodType, apiUrl },
							bufferedWriter);
					bufferedWriter.newLine();
					apiDesc = "";
					apiMethodType = "";
					apiUrl = "";
					outputApiLine = false;
					bufferedWriter.flush();
				}
			}
		}

	}

	private static void writeLineToCSV(String[] colDatas, BufferedWriter bufferedWriter) throws IOException {
		for (int i = 0; i < colDatas.length; i++) {
			String value = colDatas[i];
			bufferedWriter.write("\t" + value + " ");
			if (i < colDatas.length) {
				bufferedWriter.write(",");
			}
		}
	}

	public static void main(String[] args) throws Exception {
		genConfluenceDocs("bpm", "流程接口");
		genConfluenceDocs("portal", "门户接口");
		genConfluenceDocs("form", "表单接口");
		genConfluenceDocs("uc", "用户中心接口");
		getListOfApiUrls("bpm");
		getListOfApiUrls("portal");
		getListOfApiUrls("form");
		getListOfApiUrls("uc");
		
		// 如果是Map类型参数，需要如下示例添加ApiImplicitParam注解
//		@ApiOperation(value = "获取子组织ID（包含自己）", httpMethod = "POST", notes = "获取子组织ID（包含自己）")
//		@ApiImplicitParam(name = "ids" , paramType = "body", dataType = "Map")
//		public Map<String, Set<String>> getChildrenIds(@ApiParam(name = "ids", value = "组织id", required = true) @RequestParam Map<String,String> ids)throws Exception {
//			return orgService.getChildrenIds(ids);
//		}
	}
}
